#include "gen_exported.h"

namespace gen_exported {

/*******************************************************************************************************************
Cycling '74 License for Max-Generated Code for Export
Copyright (c) 2016 Cycling '74
The code that Max generates automatically and that end users are capable of exporting and using, and any
  associated documentation files (the “Software”) is a work of authorship for which Cycling '74 is the author
  and owner for copyright purposes.  A license is hereby granted, free of charge, to any person obtaining a
  copy of the Software (“Licensee”) to use, copy, modify, merge, publish, and distribute copies of the Software,
  and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
The Software is licensed to Licensee only for non-commercial use. Users who wish to make commercial use of the
  Software must contact the copyright owner to determine if a license for commercial use is available, and the
  terms and conditions for same, which may include fees or royalties. For commercial use, please send inquiries
  to licensing (at) cycling74.com.  The determination of whether a use is commercial use or non-commercial use is based
  upon the use, not the user. The Software may be used by individuals, institutions, governments, corporations, or
  other business whether for-profit or non-profit so long as the use itself is not a commercialization of the
  materials or a use that generates or is intended to generate income, revenue, sales or profit.
The above copyright notice and this license shall be included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
  THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*******************************************************************************************************************/

// global noise generator
Noise noise;
static const int GENLIB_LOOPCOUNT_BAIL = 100000;


// The State struct contains all the state and procedures for the gendsp kernel
typedef struct State {
	CommonState __commonstate;
	Phasor __m_phasor_10;
	int vectorsize;
	int __exception;
	t_sample m_y_1;
	t_sample m_phase_7;
	t_sample m_tone_8;
	t_sample m_rate_9;
	t_sample samplerate;
	t_sample m_depth_6;
	t_sample samples_to_seconds;
	t_sample m_shape_5;
	t_sample m_y_2;
	t_sample m_smth_3;
	t_sample m_smth_4;
	// re-initialize all member variables;
	inline void reset(t_param __sr, int __vs) {
		__exception = 0;
		vectorsize = __vs;
		samplerate = __sr;
		m_y_1 = ((int)0);
		m_y_2 = ((int)0);
		m_smth_3 = ((int)0);
		m_smth_4 = ((int)0);
		m_shape_5 = ((t_sample)0.5);
		m_depth_6 = ((int)100);
		m_phase_7 = ((int)0);
		m_tone_8 = ((int)1000);
		m_rate_9 = ((int)4);
		samples_to_seconds = (1 / samplerate);
		__m_phasor_10.reset(0);
		genlib_reset_complete(this);
		
	};
	// the signal processing routine;
	inline int perform(t_sample ** __ins, t_sample ** __outs, int __n) {
		vectorsize = __n;
		const t_sample * __in1 = __ins[0];
		const t_sample * __in2 = __ins[1];
		t_sample * __out1 = __outs[0];
		t_sample * __out2 = __outs[1];
		if (__exception) {
			return __exception;
			
		} else if (( (__in1 == 0) || (__in2 == 0) || (__out1 == 0) || (__out2 == 0) )) {
			__exception = GENLIB_ERR_NULL_BUFFER;
			return __exception;
			
		};
		t_sample expr_1090 = (((m_tone_8 * ((int)2)) * ((t_sample)3.1415926535898)) * ((t_sample)2.0833333333333e-05));
		t_sample expr_1089 = (((t_sample)0.0027777777777778) * m_phase_7);
		t_sample wrap_3 = wrap(expr_1089, ((int)0), ((int)1));
		t_sample sin_24 = sin(expr_1090);
		t_sample clamp_25 = ((sin_24 <= ((t_sample)1e-05)) ? ((t_sample)1e-05) : ((sin_24 >= ((t_sample)0.99999)) ? ((t_sample)0.99999) : sin_24));
		t_sample mul_15 = (m_depth_6 * ((t_sample)0.01));
		t_sample mul_859 = (m_depth_6 * ((t_sample)0.005));
		t_sample add_667 = (mul_859 + ((int)1));
		samples_to_seconds = (1 / samplerate);
		// the main sample loop;
		while ((__n--)) {
			const t_sample in1 = (*(__in1++));
			const t_sample in2 = (*(__in2++));
			t_sample mix_1125 = (m_shape_5 + (((t_sample)0.999) * (m_smth_4 - m_shape_5)));
			t_sample mix_1 = mix_1125;
			t_sample mix_1126 = (wrap_3 + (((t_sample)0.999) * (m_smth_3 - wrap_3)));
			t_sample mix_2 = mix_1126;
			t_sample mix_1127 = (m_y_2 + (clamp_25 * (in1 - m_y_2)));
			t_sample mix_1062 = mix_1127;
			t_sample mix_1128 = (m_y_1 + (clamp_25 * (in2 - m_y_1)));
			t_sample mix_1049 = mix_1128;
			t_sample sub_932 = (in1 - mix_1062);
			t_sample sub_1036 = (in2 - mix_1049);
			t_sample phasor_6 = __m_phasor_10(m_rate_9, samples_to_seconds);
			t_sample triangle_7 = triangle(phasor_6, mix_1);
			t_sample mul_958 = (mix_1062 * triangle_7);
			t_sample rsub_17 = (((int)1) - triangle_7);
			t_sample mul_971 = (sub_932 * rsub_17);
			t_sample add_984 = (mul_958 + mul_971);
			t_sample mix_1129 = (in1 + (mul_15 * (add_984 - in1)));
			t_sample mul_680 = (mix_1129 * add_667);
			t_sample out1 = mul_680;
			t_sample triangle_5 = triangle((mix_2 + phasor_6), mix_1);
			t_sample mul_1023 = (mix_1049 * triangle_5);
			t_sample rsub_9 = (((int)1) - triangle_5);
			t_sample mul_1010 = (sub_1036 * rsub_9);
			t_sample add_997 = (mul_1023 + mul_1010);
			t_sample mix_1130 = (in2 + (mul_15 * (add_997 - in2)));
			t_sample mul_727 = (mix_1130 * add_667);
			t_sample out2 = mul_727;
			t_sample smth1_next_1091 = fixdenorm(mix_1);
			t_sample smth2_next_1092 = fixdenorm(mix_2);
			t_sample y0_next_1093 = fixdenorm(mix_1062);
			t_sample y1_next_1094 = fixdenorm(mix_1049);
			m_smth_4 = smth1_next_1091;
			m_smth_3 = smth2_next_1092;
			m_y_2 = y0_next_1093;
			m_y_1 = y1_next_1094;
			// assign results to output buffer;
			(*(__out1++)) = out1;
			(*(__out2++)) = out2;
			
		};
		return __exception;
		
	};
	inline void set_shape(t_param _value) {
		m_shape_5 = (_value < 0.01 ? 0.01 : (_value > 0.99 ? 0.99 : _value));
	};
	inline void set_depth(t_param _value) {
		m_depth_6 = (_value < 0 ? 0 : (_value > 100 ? 100 : _value));
	};
	inline void set_phase(t_param _value) {
		m_phase_7 = (_value < -180 ? -180 : (_value > 180 ? 180 : _value));
	};
	inline void set_tone(t_param _value) {
		m_tone_8 = (_value < 500 ? 500 : (_value > 6000 ? 6000 : _value));
	};
	inline void set_rate(t_param _value) {
		m_rate_9 = (_value < 0.1 ? 0.1 : (_value > 20 ? 20 : _value));
	};
	
} State;


///
///	Configuration for the genlib API
///

/// Number of signal inputs and outputs

int gen_kernel_numins = 2;
int gen_kernel_numouts = 2;

int num_inputs() { return gen_kernel_numins; }
int num_outputs() { return gen_kernel_numouts; }
int num_params() { return 5; }

/// Assistive lables for the signal inputs and outputs

const char *gen_kernel_innames[] = { "in1", "in2" };
const char *gen_kernel_outnames[] = { "out1", "out2" };

/// Invoke the signal process of a State object

int perform(CommonState *cself, t_sample **ins, long numins, t_sample **outs, long numouts, long n) {
	State* self = (State *)cself;
	return self->perform(ins, outs, n);
}

/// Reset all parameters and stateful operators of a State object

void reset(CommonState *cself) {
	State* self = (State *)cself;
	self->reset(cself->sr, cself->vs);
}

/// Set a parameter of a State object

void setparameter(CommonState *cself, long index, t_param value, void *ref) {
	State *self = (State *)cself;
	switch (index) {
		case 0: self->set_depth(value); break;
		case 1: self->set_phase(value); break;
		case 2: self->set_rate(value); break;
		case 3: self->set_shape(value); break;
		case 4: self->set_tone(value); break;
		
		default: break;
	}
}

/// Get the value of a parameter of a State object

void getparameter(CommonState *cself, long index, t_param *value) {
	State *self = (State *)cself;
	switch (index) {
		case 0: *value = self->m_depth_6; break;
		case 1: *value = self->m_phase_7; break;
		case 2: *value = self->m_rate_9; break;
		case 3: *value = self->m_shape_5; break;
		case 4: *value = self->m_tone_8; break;
		
		default: break;
	}
}

/// Get the name of a parameter of a State object

const char *getparametername(CommonState *cself, long index) {
	if (index >= 0 && index < cself->numparams) {
		return cself->params[index].name;
	}
	return 0;
}

/// Get the minimum value of a parameter of a State object

t_param getparametermin(CommonState *cself, long index) {
	if (index >= 0 && index < cself->numparams) {
		return cself->params[index].outputmin;
	}
	return 0;
}

/// Get the maximum value of a parameter of a State object

t_param getparametermax(CommonState *cself, long index) {
	if (index >= 0 && index < cself->numparams) {
		return cself->params[index].outputmax;
	}
	return 0;
}

/// Get parameter of a State object has a minimum and maximum value

char getparameterhasminmax(CommonState *cself, long index) {
	if (index >= 0 && index < cself->numparams) {
		return cself->params[index].hasminmax;
	}
	return 0;
}

/// Get the units of a parameter of a State object

const char *getparameterunits(CommonState *cself, long index) {
	if (index >= 0 && index < cself->numparams) {
		return cself->params[index].units;
	}
	return 0;
}

/// Get the size of the state of all parameters of a State object

size_t getstatesize(CommonState *cself) {
	return genlib_getstatesize(cself, &getparameter);
}

/// Get the state of all parameters of a State object

short getstate(CommonState *cself, char *state) {
	return genlib_getstate(cself, state, &getparameter);
}

/// set the state of all parameters of a State object

short setstate(CommonState *cself, const char *state) {
	return genlib_setstate(cself, state, &setparameter);
}

/// Allocate and configure a new State object and it's internal CommonState:

void *create(t_param sr, long vs) {
	State *self = new State;
	self->reset(sr, vs);
	ParamInfo *pi;
	self->__commonstate.inputnames = gen_kernel_innames;
	self->__commonstate.outputnames = gen_kernel_outnames;
	self->__commonstate.numins = gen_kernel_numins;
	self->__commonstate.numouts = gen_kernel_numouts;
	self->__commonstate.sr = sr;
	self->__commonstate.vs = vs;
	self->__commonstate.params = (ParamInfo *)genlib_sysmem_newptr(5 * sizeof(ParamInfo));
	self->__commonstate.numparams = 5;
	// initialize parameter 0 ("m_depth_6")
	pi = self->__commonstate.params + 0;
	pi->name = "depth";
	pi->paramtype = GENLIB_PARAMTYPE_FLOAT;
	pi->defaultvalue = self->m_depth_6;
	pi->defaultref = 0;
	pi->hasinputminmax = false;
	pi->inputmin = 0;
	pi->inputmax = 1;
	pi->hasminmax = true;
	pi->outputmin = 0;
	pi->outputmax = 100;
	pi->exp = 0;
	pi->units = "";		// no units defined
	// initialize parameter 1 ("m_phase_7")
	pi = self->__commonstate.params + 1;
	pi->name = "phase";
	pi->paramtype = GENLIB_PARAMTYPE_FLOAT;
	pi->defaultvalue = self->m_phase_7;
	pi->defaultref = 0;
	pi->hasinputminmax = false;
	pi->inputmin = 0;
	pi->inputmax = 1;
	pi->hasminmax = true;
	pi->outputmin = -180;
	pi->outputmax = 180;
	pi->exp = 0;
	pi->units = "";		// no units defined
	// initialize parameter 2 ("m_rate_9")
	pi = self->__commonstate.params + 2;
	pi->name = "rate";
	pi->paramtype = GENLIB_PARAMTYPE_FLOAT;
	pi->defaultvalue = self->m_rate_9;
	pi->defaultref = 0;
	pi->hasinputminmax = false;
	pi->inputmin = 0;
	pi->inputmax = 1;
	pi->hasminmax = true;
	pi->outputmin = 0.1;
	pi->outputmax = 20;
	pi->exp = 0;
	pi->units = "";		// no units defined
	// initialize parameter 3 ("m_shape_5")
	pi = self->__commonstate.params + 3;
	pi->name = "shape";
	pi->paramtype = GENLIB_PARAMTYPE_FLOAT;
	pi->defaultvalue = self->m_shape_5;
	pi->defaultref = 0;
	pi->hasinputminmax = false;
	pi->inputmin = 0;
	pi->inputmax = 1;
	pi->hasminmax = true;
	pi->outputmin = 0.01;
	pi->outputmax = 0.99;
	pi->exp = 0;
	pi->units = "";		// no units defined
	// initialize parameter 4 ("m_tone_8")
	pi = self->__commonstate.params + 4;
	pi->name = "tone";
	pi->paramtype = GENLIB_PARAMTYPE_FLOAT;
	pi->defaultvalue = self->m_tone_8;
	pi->defaultref = 0;
	pi->hasinputminmax = false;
	pi->inputmin = 0;
	pi->inputmax = 1;
	pi->hasminmax = true;
	pi->outputmin = 500;
	pi->outputmax = 6000;
	pi->exp = 0;
	pi->units = "";		// no units defined
	
	return self;
}

/// Release all resources and memory used by a State object:

void destroy(CommonState *cself) {
	State *self = (State *)cself;
	genlib_sysmem_freeptr(cself->params);
		
	delete self;
}


} // gen_exported::
